//============================================================================
// Name        : gewoopi.cpp
// Author      : Thomas Geymayer
// Version     :
// Copyright   : (c) copyright 2008 Thomas Geymayer
// Description : Hello World in C, Ansi-style
//============================================================================

#include <iostream>

#include <FTGL/ftgl.h>

#include <gewoopi/gewoopi.hpp>
#include <gewoopi/input.hpp>

#include <gewoopi/video/renderer.h>
#include <gewoopi/scene/mesh.hpp>
#include <gewoopi/scene/manager.h>
#include <gewoopi/scene/camera_perspective.h>

using namespace gewoopi;

int main( int argc, char *argv[] )
{
  core::Application demo("gewoopi_demo",argc,argv);

  LOG_INFO("Start demo.");

  // input
  input::Button quit( demo.getInputManager()->getButton("quit") );

  input::Button left ( demo.getInputManager()->getButton("left" ) );
  input::Button right( demo.getInputManager()->getButton("right") );

  input::Button button_add( demo.getInputManager()->getButton("add") );
  input::Button button_sub( demo.getInputManager()->getButton("sub") );

  // add a few objects
  scene::Tree scene_tree = demo.getSceneManager()->getSceneTree();
  scene::Tree::iterator root = scene_tree.root();

  root.append_child( scene::Node() )->addMesh( scene::Mesh::createCube(1.5) );
  root.append_child( scene::Node( math::Vector3f(0,-5.75,0) ) )
                             ->addMesh( scene::Mesh::createCube(10) );
  scene::Tree::iterator cube = root.append_child( scene::Node( math::Vector3f(2,0,0) ) );
  cube->addMesh( scene::Mesh::createCube(0.5) );

  // create a perspective camera
  scene::CameraPerspectivePtr cam(
    new scene::CameraPerspective( math::Degreef(45),    // field of view
                                  4./3,                 // aspect ratio,
                                  0.7,                  // near clipping
                                  40 ) );               // far clipping

  video::RendererPtr renderer = demo.getRenderer();
  renderer->setCamera( cam );

  FTGLPixmapFont font("gotham_nights.ttf");

  if( font.Error() )
    std::cerr << "Font" << std::endl;

  font.FaceSize(36);

  while( demo.run() && !quit )
  {
    //--------------------------------------------------------------------------

    cube->addPosition( math::Vector3f( 0.1f * (left - right), 0, 0 ) );

    //--------------------------------------------------------------------------
    // some zooming and moving of the camera

    float scale = (cos(1.5*demo.getTimeFromStart()+2)+1) * (sin(0.81*demo.getTimeFromStart()-0.1)+1);

    cam->setFOV( cam->getFOV() + 0.5 * (button_add - button_sub) );
    cam->setTransform(
      math::Matrix4x4f::LookAtLH( math::Vector3f( (2+scale)*sin(demo.getTimeFromStart()),
                                                  1,
                                                  (2+scale)*cos(demo.getTimeFromStart())),
                                  math::Vector3f(0,0,0),
                                  math::Vector3f(0,1,0) ) );

      font.Render( ("FPS: " + utilities::toStr(demo.getFPS())).c_str(), -1, FTPoint(5,600-5-font.BBox("T").Upper().Y()) );
  }

  std::cout << boost::thread::hardware_concurrency() << std::endl;

  return 0;
}
