//------------------------------------------------------------------------------
// virtual_file_system.hpp - this file is part of the gewoopi game engine
// (c) copyright 2009 Thomas Geymayer <tomgey@gmail.com>
// For conditions of distribution and use, see copyright notice in gewoopi.h
//------------------------------------------------------------------------------

/*!
 * @file
 * @brief
 * @details
 * @author Thomas Geymayer <tomgey@gmail.com>
 * @date Date of Creation: 09.10.2009
 */

#ifndef VIRTUAL_FILE_SYSTEM_HPP_
#define VIRTUAL_FILE_SYSTEM_HPP_

#include "../std.hpp"
#include "../../utilities/fwd_default.h"

namespace gewoopi
{
namespace core
{
namespace vfs       /// Virtual File System
{

  DEF_SMART_PTR_FORWARD(class,VirtualFileSystem);

  class VirtualFileSystem
  {
    public:

      //------------------------------------------------------------------------
      ///
      /// @brief Returns the singleton instance of the virtual file system.
      ///
      /// Because the underlying library doesn't support multiple virtual file
      /// systems we can not allow to create multiple instances of this class.
      ///
      /// @return The one and only instance of this class

      static VirtualFileSystemPtr instance();

      //------------------------------------------------------------------------
      ///
      /// @brief Initializes the VirtualFileSystem singleton instance.
      ///
      /// @param argv0 If available pass the argv[0] command line argument.
      ///
      /// @note From the docs:
      /// argv0 the argv[0] string passed to your program's mainline. This may
      /// be NULL on most platforms (such as ones without a standard main()
      /// function), but you should always try to pass something in here.
      /// Unix-like systems such as Linux _need_ to pass argv[0] from main() in
      /// here.
      ///
      /// @exception std::runtime_error Thrown if an error occurs.

      static void init( const char* argv0 );

      //------------------------------------------------------------------------
      ///
      /// @brief
      ///

      ~VirtualFileSystem();

      //------------------------------------------------------------------------
      ///
      /// @brief Mount a folder or an archive
      ///

      void mount( const path& real_path,
                  const core::string& mount_point = "/" );

      //------------------------------------------------------------------------
      ///
      /// @brief Set the write directory
      ///

      void setWriteDir( const path& write_path );

      //------------------------------------------------------------------------
      ///
      /// @brief
      ///
      /// @return

      bool exists( const core::string& check_path ) const;

      //------------------------------------------------------------------------
      ///
      /// @brief
      ///
      /// @return

      const path getUserDir() const;
      const path getBaseDir() const;

    private:

      //------------------------------------------------------------------------

      static VirtualFileSystemPtr _singleton;

      //------------------------------------------------------------------------
      ///
      /// @brief Make this a singleton
      ///

      VirtualFileSystem( const char* argv0 );

  };

} // namespace vfs
} // namespace core
} // namespace gewoopi

#endif /* VIRTUAL_FILE_SYSTEM_HPP_ */
