//------------------------------------------------------------------------------
// base_button.hpp - this file is part of the gewoopi game engine
// (c) copyright 2009 Thomas Geymayer <tomgey@gmail.com>
// For conditions of distribution and use, see copyright notice in gewoopi.h
//------------------------------------------------------------------------------

/*!
 * @file
 * @brief
 * @details
 * @author Thomas Geymayer <tomgey@gmail.com>
 * @date Date of Creation: 14.10.2009
 */

#ifndef GEWOOPI_BASE_BUTTON_HPP_
#define GEWOOPI_BASE_BUTTON_HPP_

#include "../utilities/safe_bool.hpp"
#include "../core/std.hpp"
#include "fwd.h"

namespace gewoopi
{
namespace input
{

  class BaseButton: public utilities::safe_bool<BaseButton>
  {
    public:

      typedef core::real32 ValueType;

      //------------------------------------------------------------------------
      ///
      /// @brief Constructor
      ///
      /// @param name The name of this button

      BaseButton( const core::string& name = "unknown" );

      //------------------------------------------------------------------------
      ///
      /// @brief Destructor
      ///

      virtual ~BaseButton();

      // All methods regarding the _value member handle the parameters and
      // return values per value because this class is meant to be used with
      // integral data types where it would just get slower with references.

      //------------------------------------------------------------------------
      ///
      /// @brief Get the value of the button
      ///
      /// @return The current value of the button

      virtual const ValueType getValue() const = 0;

      //------------------------------------------------------------------------
      ///
      /// @brief Get the boolean value of the button
      ///
      /// @return The current value of the button

      virtual const bool isPressed() const;

      //------------------------------------------------------------------------
      ///
      /// @brief Cast operator to get the value of the button
      ///
      /// @return The current value of the button

      virtual operator const ValueType() const;
      virtual const bool operator_bool() const;

      //------------------------------------------------------------------------
      ///
      /// @brief Get the name
      ///
      /// @return The name

      virtual const core::string& getName() const;

      //------------------------------------------------------------------------
      ///
      /// @brief Set the name
      ///
      /// @param name the new name

      virtual void setName( const core::string& name );

    protected:

      core::string  _name;

  };

} // namespace input
} // namespace gewoopi

#endif /* GEWOOPI_BASE_BUTTON_HPP_ */
