//------------------------------------------------------------------------------
// button.h - this file is part of the gewoopi game engine
// (c) copyright 2009 Thomas Geymayer <tomgey@gmail.com>
// For conditions of distribution and use, see copyright notice in gewoopi.h
//------------------------------------------------------------------------------

/*!
 * @file: button.h
 * @brief
 * @details
 * @author Thomas Geymayer <tomgey@gmail.com>
 * @date Date of Creation: 14.07.2009
 */

#ifndef INPUT_BUTTON_H_
#define INPUT_BUTTON_H_

#include "base_button.hpp"

namespace gewoopi
{
namespace input
{

  class Button: public BaseButton
  {

    public:

      //------------------------------------------------------------------------
      ///
      /// @brief Constructor
      ///
      /// @param name The name of this button

      Button( const core::string& name = "unknown" );

      //------------------------------------------------------------------------
      ///
      /// @brief Destructor
      ///

      virtual ~Button();

      //------------------------------------------------------------------------
      ///
      /// @brief Add a physical button
      ///

      void add( PhysicalButtonPtr button );

      // All methods regarding the _value member handle the parameters and
      // return values per value because this class is meant to be used with
      // integral data types where it would just get slower with references.

      //------------------------------------------------------------------------
      ///
      /// @brief Get the value of the button
      ///
      /// @return The current value of the button

      virtual const BaseButton::ValueType getValue() const;

    protected:

      std::vector<PhysicalButtonPtr> _buttons;

  };

} // namespace input
} // namespace gewoopi

#endif /* INPUT_BUTTON_H_ */
