//------------------------------------------------------------------------------
// device.h - this file is part of the gewoopi game engine
// (c) copyright 2009 Thomas Geymayer <tomgey@gmail.com>
// For conditions of distribution and use, see copyright notice in gewoopi.h
//------------------------------------------------------------------------------

/*!
 * @file: device.h
 * @brief
 * @details
 * @author Thomas Geymayer <tomgey@gmail.com>
 * @date Date of Creation: 05.07.2009
 */

#ifndef INPUT_DEVICE_H_
#define INPUT_DEVICE_H_

#include "../core/std.hpp"
#include "fwd.h"

namespace gewoopi
{
namespace input
{

  class Device
  {
    public:

      enum Types
      {
        KEYBOARD,
        MOUSE,
        JOYSTICK,
        UNKNOWN
      };

      //------------------------------------------------------------------------
      ///
      /// @brief Constructor
      ///

      Device( Types type, const core::string& name, size_t num_buttons );

      //------------------------------------------------------------------------
      ///
      /// @brief Destructor
      ///

      ~Device();

      //------------------------------------------------------------------------
      ///
      /// @brief Get a button
      ///
      /// Get the button with the specified index on this this. The index must
      /// be in the range from 0 to _num_buttons, which has been specified
      /// in the constructor
      ///
      /// @return Reference to the button

      /// @note The buttons are returned per reference because in this way the
      ///       overloaded operators of the Button class can be used in an easy
      ///       and intuitive way, eg:
      ///       @code move = keyboard.getButton(5) * speed; @endcode

      PhysicalButtonPtr getButton( size_t index );

      // TODO also add device id as parameter? (different devices of the same
      // type probably have different number of buttons)

      static size_t getNumButtons( Types type );
      static size_t getId( Types type, const core::string& name );

    private:

      //------------------------------------------------------------------------

      core::string _name;

      size_t                                    _num_buttons;
      boost::shared_array<PhysicalButtonPtr>    _buttons;

      static const char** getButtonNames( Types type );

  };

} // namespace input
} // namespace gewoopi

#endif /* INPUT_DEVICE_H_ */
