//------------------------------------------------------------------------------
// physical_button.hpp - this file is part of the gewoopi game engine
// (c) copyright 2009 Thomas Geymayer <tomgey@gmail.com>
// For conditions of distribution and use, see copyright notice in gewoopi.h
//------------------------------------------------------------------------------

/*!
 * @file
 * @brief
 * @details
 * @author Thomas Geymayer <tomgey@gmail.com>
 * @date Date of Creation: 14.10.2009
 */

#ifndef GEWOOPI_PHYSICAL_BUTTON_HPP_
#define GEWOOPI_PHYSICAL_BUTTON_HPP_

#include "base_button.hpp"

namespace gewoopi
{
namespace input
{

  class PhysicalButton: public BaseButton
  {
    public:

      //------------------------------------------------------------------------
      ///
      /// @brief Constructor
      ///
      /// @param name The name of this button
      /// @param value The initial value of the button

      PhysicalButton( const core::string& name = "unknown",
                      ValueType value = ValueType() );

      //------------------------------------------------------------------------
      ///
      /// @brief Destructor
      ///

      virtual ~PhysicalButton();

      // All methods regarding the _value member handle the parameters and
      // return values per value because this class is meant to be used with
      // integral data types where it would just get slower with references.

      //------------------------------------------------------------------------
      ///
      /// @brief Get the value of the button
      ///
      /// @return The current value of the button

      virtual const ValueType getValue() const;

      //------------------------------------------------------------------------
      ///
      /// @brief Set the value of the button
      ///
      /// @param value the new value

      virtual void setValue( const ValueType value );

      //------------------------------------------------------------------------
      ///
      /// @brief Add value to the current value of the button
      ///
      /// @param value the value to add

      virtual void addValue( const ValueType value );

    protected:

      ValueType     _value;

  };

} // namespace input
} // namespace gewoopi

#endif /* GEWOOPI_PHYSICAL_BUTTON_HPP_ */
