//------------------------------------------------------------------------------
// angle.hpp - this file is part of the gewoopi game engine
// (c) copyright 2009 Thomas Geymayer <tomgey@gmail.com>
// For conditions of distribution and use, see copyright notice in gewoopi.h
//------------------------------------------------------------------------------

/*!
 * @file
 * @brief
 * @details
 * @author Thomas Geymayer <tomgey@gmail.com>
 * @date Date of Creation: 26.10.2009
 */

#ifndef GEWOOPI_MATH_ANGLE_HPP_
#define GEWOOPI_MATH_ANGLE_HPP_

#include "constants.hpp"

namespace gewoopi
{
namespace math
{

  // forward declaration for an angle in radian
  template<typename T> class Radian;

  //----------------------------------------------------------------------------
  ///
  /// Degree
  ///
  /// @brief An angle in degree
  /// @details
  ///
  /// @author <Thomas Geymayer tomgey@gmail.com>

  template<typename T>
  class Degree
  {
    public:

      //------------------------------------------------------------------------
      ///
      /// Default constructor
      ///

      explicit Degree( const T degree = 0 ):
        _value( degree )
      {}

      // implementation follows after definition of Radian
      Degree( const Radian<T> radian );

      operator const T() const
      { return _value; }

      const T getValue() const
      { return _value; }

    private:

      //------------------------------------------------------------------------

      T _value;

  };

  //----------------------------------------------------------------------------
  ///
  /// Radian
  ///
  /// @brief An angle in radian
  /// @details
  ///
  /// @author <Thomas Geymayer tomgey@gmail.com>

  template<typename T>
  class Radian
  {
    public:

      //------------------------------------------------------------------------
      ///
      /// Default constructor
      ///

      explicit Radian( const T radian = 0 ):
        _value( radian )
      {}

      Radian( const Degree<T> degree ):
        _value( degree * pi<T>()/180. )
      {}

      operator const T() const
      { return _value; }

      const T getValue() const
      { return _value; }

    private:

      //------------------------------------------------------------------------

      T _value;

  };

  template<typename T>
  Degree<T>::Degree( const Radian<T> radian ):
    _value( radian * 180/pi<T>() )
  {}

} // namespace math
} // namespace gewoopi

#endif /* GEWOOPI_MATH_ANGLE_HPP_ */
