//------------------------------------------------------------------------------
// face.hpp - this file is part of the gewoopi game engine
// (c) copyright 2009 Thomas Geymayer <tomgey@gmail.com>
// For conditions of distribution and use, see copyright notice in gewoopi.h
//------------------------------------------------------------------------------

/*!
 * @file
 * @brief
 * @details
 * @author Thomas Geymayer <tomgey@gmail.com>
 * @date Date of Creation: 17.10.2009
 */

#ifndef GEWOOPI_FACE_HPP_
#define GEWOOPI_FACE_HPP_

#include "plane.hpp"

namespace gewoopi
{
namespace math
{

  class Face
  {
    public:

      //------------------------------------------------------------------------
      ///
      /// Create a new Face from three given points
      ///

      Face( const Vector3f& p0, const Vector3f& p1, const Vector3f& p2 );

      //------------------------------------------------------------------------
      ///
      /// Create an uninitialized Face
      ///

      Face();

      //------------------------------------------------------------------------
      ///
      /// Check for a collision
      ///
      /// @param ray      The ray to check the collision with
      /// @param distance If not zero the distance is stored at it's address
      /// @param normal   If not zero the normal at the hitpoint is stored at
      ///                 it's address

      bool hit( const Ray3f& ray,
                real* distance   = NULL,
                Vector3f* normal = NULL ) const;

    private:

      //------------------------------------------------------------------------

      Plane  _main_plane;
      Plane  _margin_planes[3];

  };

  //----------------------------------------------------------------------------

  typedef Face Facef; ///< Just make it conformant to the naming scheme

} // namespace math
} // namespace gewoopi

#endif /* GEWOOPI_FACE_HPP_ */
