//------------------------------------------------------------------------------
// ray.hpp - this file is part of the gewoopi game engine
// (c) copyright 2009 Thomas Geymayer <tomgey@gmail.com>
// For conditions of distribution and use, see copyright notice in gewoopi.h
//------------------------------------------------------------------------------

/*!
 * @file
 * @brief
 * @details
 * @author Thomas Geymayer <tomgey@gmail.com>
 * @date Date of Creation: 20.10.2009
 */

#ifndef GEWOOPI_MATH_RAY_HPP_
#define GEWOOPI_MATH_RAY_HPP_

#include "vector.hpp"

namespace gewoopi
{
namespace math
{

  //----------------------------------------------------------------------------
  ///
  /// Ray
  ///
  /// @brief Represents a ray
  /// @details
  ///
  /// @author <Thomas Geymayer tomgey@gmail.com>

  template<typename VectorType>
  struct Ray
  {
    //--------------------------------------------------------------------------
    ///
    /// Default constructor
    ///
    /// @param origin     The origin of the ray
    /// @param direction  The direction of the ray

    Ray( const VectorType& origin,
         const VectorType& direction ):
      _origin( origin ),
      _direction( direction )
    {}

    //--------------------------------------------------------------------------

    VectorType _origin;
    VectorType _direction;

  };

  typedef Ray<Vector2f> Ray2f; ///< Ray in 2d space
  typedef Ray<Vector3f> Ray3f; ///< Ray in 3d space

} // namespace math
} // namespace gewoopi

#endif /* GEWOOPI_MATH_RAY_HPP_ */
