//------------------------------------------------------------------------------
// types.hpp - this file is part of the gewoopi game engine
// (c) copyright 2009 Thomas Geymayer <tomgey@gmail.com>
// For conditions of distribution and use, see copyright notice in gewoopi.h
//------------------------------------------------------------------------------

/*!
 * @file
 * @brief
 * @details
 * @author Thomas Geymayer <tomgey@gmail.com>
 * @date Date of Creation: 24.09.2009
 */

#ifndef GEWOOPI_MATH_TYPES_HPP_
#define GEWOOPI_MATH_TYPES_HPP_

#include <boost/cstdint.hpp>
#include "angle.hpp"

namespace gewoopi
{
namespace math
{

  //----------------------------------------------------------------------------
  // Real number types

  typedef float     real32;
  typedef double    real64;

#ifdef GEWOOPI_REAL_DOUBLE_PRECISSION
  typedef real64t   real;
#else
  typedef real32    real;
#endif

  //----------------------------------------------------------------------------
  // Integer types

  // signed
  typedef int8_t    int8;
  typedef int16_t   int16;
  typedef int32_t   int32;
  typedef int64_t   int64;

  // unsigned
  typedef uint8_t   uint8;
  typedef uint16_t  uint16;
  typedef uint32_t  uint32;
  typedef uint64_t  uint64;

  //----------------------------------------------------------------------------
  // angles

  typedef Radian<real32>    Radianf;
  typedef Degree<real32>    Degreef;

} // namespace math
} // namespace gewoopi

#endif /* GEWOOPI_MATH_TYPES_HPP_ */
