//------------------------------------------------------------------------------
// scene_manager.h - this file is part of the gewoopi game engine
// (c) copyright 2008 Thomas Geymayer <tomgey@gmail.com>
// For conditions of distribution and use, see copyright notice in gewoopi.h
//------------------------------------------------------------------------------

#ifndef SCENE_MANAGER_H_
#define SCENE_MANAGER_H_

#include <tp-tree/tree.h>

#include "fwd.h"
#include "node.h"
#include "../video/fwd.h"

namespace gewoopi
{
namespace scene
{

  typedef tp::tree<Node> Tree;

  class Manager
  {
    public:
      Manager();
      virtual ~Manager();

      video::RendererWeakPtr setRenderer( video::RendererWeakPtr renderer );

      void render( double run_time, double frame_time );
      void move(   double run_time, double frame_time );

      Tree getSceneTree();

      //------------------------------------------------------------------------
      ///
      /// @brief Create a new scene node
      ///
      /// This node is appended to the scene tree root node.
      ///
      /// @param position The position of the node
      /// @param rotation The rotation of the node
      ///
      /// @return Iterator to the new node

      Tree::iterator
      createNode( const math::Vector3f& position = math::Vector3f(),
                  const math::Vector3f& rotation = math::Vector3f() );

    protected:

      video::RendererWeakPtr _renderer;
      Tree _scene_tree;
  };

} // namespace scene
} // namespace gewoopi

#endif /* SCENE_MANAGER_H_ */
