//------------------------------------------------------------------------------
// mesh.h - this file is part of the gewoopi game engine
// (c) copyright 2008 - 2009 Thomas Geymayer <tomgey@gmail.com>
// For conditions of distribution and use, see copyright notice in gewoopi.h
//------------------------------------------------------------------------------

#ifndef GEWOOPI_MESH_HPP_
#define GEWOOPI_MESH_HPP_

#include <boost/shared_array.hpp>
#include <boost/shared_ptr.hpp>

#include <stdint.h>

#include "fwd.h"
#include "../video/fwd.h"
#include "../video/vertex.h"
#include "../math/vector.hpp"
#include "../math/face.hpp"

namespace gewoopi
{
namespace scene
{

  typedef video::Vertex Vertex;
  typedef math::uint32  Count;
  typedef math::Facef Face;

  typedef boost::shared_array<Vertex>   VertexArray;
  typedef boost::shared_array<Face>     FaceArray;

  class Mesh
  {
    public:

      //------------------------------------------------------------------------
      Mesh( VertexArray vertices,
            Count vertex_count,
            const video::PrimitiveType primitive_type );

      //------------------------------------------------------------------------
      virtual ~Mesh();

      //------------------------------------------------------------------------
      ///
      /// Calculate some internal data to enable collision detection
      ///

      void calculateCollisionData();

      //------------------------------------------------------------------------
      ///
      /// Check for a collision
      ///
      /// @param ray      The ray to check the collision with
      /// @param distance If not zero the distance is stored at it's address
      /// @param normal   If not zero the normal at the hitpoint is stored at
      ///                 it's address

      bool hit( const math::Ray3f& ray,
                math::real* distance   = NULL,
                math::Vector3f* normal = NULL ) const;

      //------------------------------------------------------------------------
      ///
      /// Render the mesh on the given renderer
      ///

      virtual void render( video::RendererPtr renderer, int pass = 0 );

      //------------------------------------------------------------------------
      ///
      /// Create a mesh representing a cube
      ///

      static MeshPtr createCube( float size = 1.f );

    protected:

      VertexArray           _vertices;
      Count                 _vertex_count;
      video::PrimitiveType  _primitive_type;

      FaceArray             _faces;
      Count                 _face_count;
  };

} // namespace gewoopi
} // namespace scene

#endif /* GEWOOPI_MESH_HPP_ */
