//------------------------------------------------------------------------------
// node.h - this file is part of the gewoopi game engine
// (c) copyright 2008 Thomas Geymayer <tomgey@gmail.com>
// For conditions of distribution and use, see copyright notice in gewoopi.h
//------------------------------------------------------------------------------

#ifndef SCENE_NODE_H_
#define SCENE_NODE_H_

#include <set>

#include "../math/matrix.hpp"
#include "mesh.hpp"

namespace gewoopi
{
namespace scene
{

  class Node
  {
    public:

      Node( const math::Vector3f& position = math::Vector3f(),
            const math::Vector3f& rotation = math::Vector3f() );
      virtual ~Node();

      virtual void render( video::RendererPtr renderer, int pass );
      virtual void move( double run_time,
                         double frame_time,
                         const math::Matrix4x4f& parent_transform =
                           math::Matrix4x4f::Identity(),
                         bool parent_changed = false );

      //------------------------------------------------------------------------
      ///
      /// @brief Returns whether the transformation of this node has changed
      ///
      /// The state of change is updated after each call to Node::move.
      ///
      /// @return true If the absolute transformation matrix has changed

      bool hasChanged();

      MeshPtr addMesh( MeshPtr mesh );

      //------------------------------------------------------------------------
      ///
      /// Check if a line hits one of the attached meshes
      ///

      bool hit( const math::Ray3f& ray,
                math::real* distance   = NULL,
                math::Vector3f* normal = NULL ) const;

      void setPosition( const math::Vector3f& position );
      void setRotation( const math::Vector3f& rotation );

      void addPosition( const math::Vector3f& move );
      void addRotation( const math::Vector3f& rotate );

      const math::Matrix4x4f& getTransformAbs() const;

    protected:

      typedef std::set<MeshPtr> MeshList;
      MeshList _meshes;

      math::Vector3f _position_rel;
      math::Vector3f _rotation_rel;

      math::Matrix4x4f _transform_abs;
      math::Matrix4x4f _transform_rel;

      bool _rel_changed;
      bool _abs_changed;
  };

} // namespace gewoopi
} // namespace scene

#endif /* NODE_H_ */
