//------------------------------------------------------------------------------
// dynamic_average.h - this file is part of the gewoopi game engine
// (c) copyright 2009 Thomas Geymayer <tomgey@gmail.com>
// For conditions of distribution and use, see copyright notice in gewoopi.h
//------------------------------------------------------------------------------

/*!
 * @file: dynamic_average.h
 * @brief
 * @details
 * @author Thomas Geymayer <tomgey@gmail.com>
 * @date Date of Creation: 14.05.2009
 */

#ifndef DYNAMIC_AVERAGE_H_
#define DYNAMIC_AVERAGE_H_

namespace gewoopi
{
namespace utilities
{

  template <typename T, unsigned short count>
  class DynamicAverage
  {
    public:
      DynamicAverage( T start_value = T() ):
        _current_index(0),
        _sum(0),
        _average(start_value)
      {
        for( unsigned int i = 0; i < count; ++i )
          _values[i] = 0;
      };

      ~DynamicAverage(){};

      T getAverage()
      {
        return _average;
      }

      void addValue( T value )
      {
        // exchange value
        _sum += value - _values[_current_index];
        _values[_current_index] = value;

        if( ++_current_index >= count )
          _current_index = 0;

        _average = _sum / count;
      }

    private:

      unsigned short _current_index;
      T _values[count];
      T _sum;
      T _average;
  };

  //----------------------------------------------------------------------------
  ///
  /// @brief Protect against use with a count of 0
  ///

  template <typename T> class DynamicAverage<T,0>
  {
    DynamicAverage();
  };

} // namespace utilities
} // namespace gewoopi

#endif /* DYNAMIC_AVERAGE_H_ */
