//------------------------------------------------------------------------------
// manager.hpp - this file is part of the gewoopi game engine
// (c) copyright 2009 Thomas Geymayer <tomgey@gmail.com>
// For conditions of distribution and use, see copyright notice in gewoopi.h
//------------------------------------------------------------------------------

/*!
 * @file
 * @brief
 * @details
 * @author Thomas Geymayer <tomgey@gmail.com>
 * @date Date of Creation: 22.09.2009
 */

#ifndef GEWOOPI_UTILITIES_MANAGER_HPP_
#define GEWOOPI_UTILITIES_MANAGER_HPP_

#include "../core/std.hpp"
#include "loader.hpp"

namespace gewoopi
{
namespace utilities
{

  //----------------------------------------------------------------------------
  ///
  /// Loader
  ///
  /// @brief Base class for file loaders for models, scenes, etc.
  /// @details
  ///
  /// @author <Thomas Geymayer tomgey@gmail.com>
  /// @version <1.0.0>
  template<typename T, typename Derived>
  class Manager:
    public singleton<Derived>
  {
    friend class singleton<Derived>;

    public:

      //------------------------------------------------------------------------

      typedef typename Loader<T>::Ptr           Ptr;
      typedef boost::shared_ptr<Loader<T> >     LoaderPtr;
      typedef std::map<core::string,LoaderPtr>  LoaderList;
      typedef typename LoaderList::iterator     LoaderListIterator;

      //------------------------------------------------------------------------
      ///
      /// @brief Load a mesh from a file
      ///
      /// @return

      Ptr load( const core::path& file )
      {
        LoaderListIterator it = _loaders.find( file.extension() );
        if( it == _loaders.end() )
        {
          std::cout << "[WARNING] No loader for *" << file.extension()
                    << " found." << std::endl;
          return Ptr();
        }
        else
        {
          // let the found loader load...
          return it->second->load(file);
        }
      }

      //------------------------------------------------------------------------
      ///
      /// @brief Add a loader for a type of objects to load
      ///
      /// @param extension  The extension (with dot) which the specified
      ///                   loader can load
      /// @param loader     The loader

      void addLoader( const core::string& extension,
                      LoaderPtr& loader )
      {
        LoaderListIterator it( _loaders.find( extension ) );

        if( it != _loaders.end() )
          std::cout << "[WARNING] Loader for *" << extension
                    << " allready set." << std::endl;

        _loaders[extension] = loader;
      }


    protected:

      LoaderList _loaders;

      //------------------------------------------------------------------------
      // make it singleton

      Manager(){}

  };

} // namespace utilities
} // namespace gewoopi

#endif /* GEWOOPI_UTILITIES_MANAGER_HPP_ */
