//------------------------------------------------------------------------------
// singleton.hpp - this file is part of the gewoopi game engine
// (c) copyright 2009 Thomas Geymayer <tomgey@gmail.com>
// For conditions of distribution and use, see copyright notice in gewoopi.h
//------------------------------------------------------------------------------

/*!
 * @file
 * @brief
 * @details
 * @author Thomas Geymayer <tomgey@gmail.com>
 * @date Date of Creation: 23.10.2009
 */

#ifndef GEWOOPI_SINGLETON_HPP_
#define GEWOOPI_SINGLETON_HPP_

#include <boost/noncopyable.hpp>

namespace gewoopi
{
namespace utilities
{

  //----------------------------------------------------------------------------
  ///
  /// A base class for singleton classes
  /// @code
  /// class MyClass: public singleton<MyClass>
  /// {
  ///   friend class singleton<MyClass>;
  ///
  ///   private:
  ///
  ///     MyClass(){}
  /// }
  /// @endcode

  template<typename Derived>
  class singleton: private boost::noncopyable
  {
    public:

      static Derived* instance()
      {
        static Derived inst;
        return &inst;
      }
  };


} // namespace utilities
} // namespace gewoopi

#endif /* GEWOOPI_SINGLETON_HPP_ */
