//------------------------------------------------------------------------------
// material.hpp - this file is part of the gewoopi game engine
// (c) copyright 2009 Thomas Geymayer <tomgey@gmail.com>
// For conditions of distribution and use, see copyright notice in gewoopi.h
//------------------------------------------------------------------------------

/*!
 * @file
 * @brief
 * @details
 * @author Thomas Geymayer <tomgey@gmail.com>
 * @date Date of Creation: 27.09.2009
 */

#ifndef MATERIAL_HPP_
#define MATERIAL_HPP_

#include "../core/std.hpp"
#include "../math/vector.hpp"

namespace gewoopi
{
namespace video
{

  class Material
  {
    public:

      Material( math::Color4f const& diffuse  = math::Color4f( 0.75, 0.61, 0.23 , 1),
                math::Color4f const& ambient  = math::Color4f( 0.25, 0.2,  0.07 , 1),
                math::Color4f const& specular = math::Color4f( 0.63, 0.65, 0.37 , 1),
                math::Color4f const& emissive = math::Color4f( 0.2, 0.2, 0.2 , 1),
                core::real32 power = 51.2 );

      math::Color4f const& getDiffuse() const;
      math::Color4f const& getAmbient() const;
      math::Color4f const& getSpecular() const;
      math::Color4f const& getEmissive() const;
      core::real32 getPower() const;

    private:

      math::Color4f   _diffuse;
      math::Color4f   _ambient;
      math::Color4f   _specular;
      math::Color4f   _emissive;
      core::real32    _power;
  };

} // namespace video
} // namespace gewoopi

#endif /* MATERIAL_HPP_ */
