//------------------------------------------------------------------------------
// window.h - this file is part of the gewoopi game engine
// (c) copyright 2008 Thomas Geymayer <tomgey@gmail.com>
// For conditions of distribution and use, see copyright notice in gewoopi.h
//------------------------------------------------------------------------------

#ifndef WINDOW_H_
#define WINDOW_H_

#include <boost/signal.hpp>
#include <boost/function.hpp>

#include "render_target.h"
#include "../core/fwd.h"
#include "../input/fwd.h"

namespace gewoopi
{
namespace video
{

  class Window: public RenderTarget
  {
    public:

      //------------------------------------------------------------------------
      ///
      /// @brief Create a new window with the specified size and format
      ///
      /// @param size The two dimensional size of the window
      /// @param format The surface format

      Window( const math::Size2D<int>& size,
              SurfaceFormat format );

      //------------------------------------------------------------------------
      ///
      /// @brief Destructor
      ///

      virtual ~Window();

      //------------------------------------------------------------------------
      ///
      ///  @brief
      ///

      void setCaption( const core::string& caption, bool force = false );

      //------------------------------------------------------------------------
      ///
      ///  @brief
      ///

      const core::string& getCaption() const;

      //------------------------------------------------------------------------
      ///
      /// @brief Processes all pending messages
      ///

      virtual void processMessages() = 0;

      //------------------------------------------------------------------------
      ///
      ///
      ///

      const bool isAlive() const;

    protected:

      bool          _alive;
      core::string  _caption;

      virtual bool updateCaption( const core::string& caption ) = 0;

  };

} // namespace video
} // namespace gewoopi

#endif /* WINDOW_H_ */
