//------------------------------------------------------------------------------
// face.cpp - this file is part of the gewoopi game engine
// (c) copyright 2009 Thomas Geymayer <tomgey@gmail.com>
// For conditions of distribution and use, see copyright notice in gewoopi.h
//------------------------------------------------------------------------------

/*!
 * @file
 * @brief
 * @details
 * @author Thomas Geymayer <tomgey@gmail.com>
 * @date Date of Creation: 20.10.2009
 */

#include "math/face.hpp"

namespace gewoopi
{
namespace math
{

  //----------------------------------------------------------------------------
  Face::Face( const Vector3f& p0, const Vector3f& p1, const Vector3f& p2 ):
    _main_plane(p0, p1, p2)
  {
    // create margin planes with the normal vectors looking in the
    // middle of the face
    _margin_planes[0] = Plane( p0, _main_plane.getNormal().cross(p2 - p0) );
    _margin_planes[1] = Plane( p1, _main_plane.getNormal().cross(p0 - p1) );
    _margin_planes[2] = Plane( p2, _main_plane.getNormal().cross(p1 - p2) );
  }

  //----------------------------------------------------------------------------
  Face::Face()
  {

  }

  //----------------------------------------------------------------------------
  bool Face::hit( const Ray3f& ray,
                  real* distance,
                  Vector3f* normal ) const
  {
    real dist;

    // first check if the main plane is hit
    if( _main_plane.hit(ray, &dist, normal) )
    {
      const Vector3f hitpoint( ray._origin + dist * ray._direction );

      // now lets check if the hitpoint is in front of all
      // margin planes. If not, it doesnt hit the Face.

      for( size_t i = 0; i < 3; ++i )
        if( _margin_planes[i].isBehind(hitpoint) )
          return false;

      // the hitpoint is in front of all margin planes...
      if( distance )
        // lets set the distance
        *distance = dist;

      return true;
    }

    return false;
  }

} // namespace math
} // namespace gewoopi
