//------------------------------------------------------------------------------
// camera.cpp - this file is part of the gewoopi game engine
// (c) copyright 2008 Thomas Geymayer <tomgey@gmail.com>
// For conditions of distribution and use, see copyright notice in gewoopi.h
//------------------------------------------------------------------------------

/*!
 * @file: camera.cpp
 * @brief
 * @details
 * @author Thomas Geymayer <tomgey@gmail.com>
 * @date Date of Creation: 03.05.2009
 */

#include "scene/camera.h"

namespace gewoopi
{
namespace scene
{

  //----------------------------------------------------------------------------
  Camera::Camera( const math::Matrix4x4f& projection,
                  const math::Matrix4x4f& transform ):
    _projection( projection ),
    _changed( true ),
    _transform( transform )
  {

  }

  //----------------------------------------------------------------------------
  Camera::~Camera()
  {

  }

  //----------------------------------------------------------------------------
  const math::Matrix4x4f& Camera::getProjection() const
  {
    // lazy update
    if( _changed )
    {
      update();
      _changed = false;
    }

    return _projection;
  }

  //----------------------------------------------------------------------------
  void Camera::setTransform( const math::Matrix4x4f& transform )
  {
    _transform = transform;
  }

  //----------------------------------------------------------------------------
  const math::Matrix4x4f& Camera::getTransform() const
  {
    return _transform;
  }

  //----------------------------------------------------------------------------
  void Camera::update() const
  {
    // Nothing to do. Should be overridden by derived classes.
  }

} // namespace scene
} // namespace gewoopi
