//------------------------------------------------------------------------------
// camera_perspective.cpp - this file is part of the gewoopi game engine
// (c) copyright 2009 Thomas Geymayer <tomgey@gmail.com>
// For conditions of distribution and use, see copyright notice in gewoopi.h
//------------------------------------------------------------------------------

/*!
 * @file: camera_perspective.cpp
 * @brief
 * @details
 * @author Thomas Geymayer <tomgey@gmail.com>
 * @date Date of Creation: 18.07.2009
 */

#include "scene/camera_perspective.h"

namespace gewoopi
{
namespace scene
{

  //----------------------------------------------------------------------------
  CameraPerspective::CameraPerspective( const math::Size2D<math::real>& size,
                                        const math::real z_near,
                                        const math::real z_far ):
    _size( size ),
    _near( z_near ),
    _far(  z_far ),
    _aspect( _size.width()/_size.height() )
  {

  }

  //----------------------------------------------------------------------------
  CameraPerspective::CameraPerspective( const math::Radian<math::real> fovy,
                                        const math::real aspect,
                                        const math::real z_near,
                                        const math::real z_far ):
    _size( 0, 2 * z_near * tan(fovy) ),
    _near( z_near ),
    _far(  z_far )
  {
    // calculate width separate because we need the height for it
    _size.width( _size.height() * aspect );

    _aspect = _size.width()/_size.height();
  }

  //----------------------------------------------------------------------------
  CameraPerspective::~CameraPerspective()
  {

  }

  //----------------------------------------------------------------------------
  void CameraPerspective::setFOV( float fovy )
  {
    _size.height(2 * _near * tan( fovy * math::pi<float>()/180) );
    _size.width( _size.height() * _aspect );

    _changed = true;
  }

  //----------------------------------------------------------------------------
  const float CameraPerspective::getFOV() const
  {
    return   180/math::pi<float>()
           * atan( 0.5 * _size.height() / _near );
  }

  //----------------------------------------------------------------------------
  void CameraPerspective::update() const
  {
    _projection = math::Matrix4x4f::PerspectiveLH( _size, _near, _far );
  }

} // namespace scene
} // namespace gewoopi
