//------------------------------------------------------------------------------
// manager.cpp - this file is part of the gewoopi game engine
// (c) copyright 2008 Thomas Geymayer <tomgey@gmail.com>
// For conditions of distribution and use, see copyright notice in gewoopi.h
//------------------------------------------------------------------------------

#include "scene/manager.h"
#include "video/renderer.h"
#include "utilities/utilities.h"

namespace gewoopi
{
namespace scene
{

  //----------------------------------------------------------------------------
  Manager::Manager():
    _renderer(),
    _scene_tree()
  {
    //_scene_tree.root().append_child( Node() );
  }

  //----------------------------------------------------------------------------
  Manager::~Manager()
  {

  }

  //----------------------------------------------------------------------------
  video::RendererWeakPtr Manager::setRenderer( video::RendererWeakPtr renderer )
  {
    return utilities::setNewReturnOld( _renderer, renderer );
  }

  //----------------------------------------------------------------------------
  void Manager::render( double run_time, double frame_time )
  {
    video::RendererPtr renderer = _renderer.lock();

    assert( renderer );

    renderer->setClearColor(  math::Color3f( 0.3 + 0.5*fabs( sin(1.1  * run_time) ),
                                             0.3 + 0.5*fabs( cos(1.3  * run_time) ),
                                             0.3 + 0.5*fabs( sin(0.95 * run_time) ) )  );

    Tree::iterator node = _scene_tree.begin();

    // default iterator is a pre order iterator, so we can ignore the first node
    // because this is the root node which doesn't need to be moved. Because
    // of the pre order it is guaranteed that all parents get updated before its
    // children.

    while( ++node != _scene_tree.end() )
    {
      node->render( renderer, 1 );
    }
  }

  //----------------------------------------------------------------------------
  void Manager::move(   double run_time, double frame_time )
  {
    Tree::iterator node = _scene_tree.begin();

    // default iterator is a pre order iterator, so we can ignore the first node
    // because this is the root node which doesn't need to be moved. Because
    // of the pre order it is guaranteed that all parents get updated before its
    // children.

    while( ++node != _scene_tree.end() )
    {
      node->move( run_time, frame_time,
                  node.parent()->getTransformAbs(),
                  node.parent()->hasChanged() );
    }
  }

  //----------------------------------------------------------------------------
  Tree Manager::getSceneTree()
  {
    return _scene_tree;
  }

  //----------------------------------------------------------------------------
  Tree::iterator Manager::createNode( const math::Vector3f& position,
                                      const math::Vector3f& rotation )
  {
    return _scene_tree.root().append_child( Node(position,rotation) );
  }

} // namespace scene
} // namespace gewoopi
