//------------------------------------------------------------------------------
// loader_obj.hpp - this file is part of the gewoopi game engine
// (c) copyright 2009 Thomas Geymayer <tomgey@gmail.com>
// For conditions of distribution and use, see copyright notice in gewoopi.h
//------------------------------------------------------------------------------

/*!
 * @file
 * @brief
 * @details
 * @author Thomas Geymayer <tomgey@gmail.com>
 * @date Date of Creation: 22.09.2009
 */

#ifndef GEWOOPI_LOADER_OBJ_HPP_
#define GEWOOPI_LOADER_OBJ_HPP_

#include <vector>

#include "utilities/loader.hpp"
#include "core/vfs/file_iterator.hpp"
#include "scene/mesh.hpp"

namespace gewoopi
{
namespace scene
{

  //----------------------------------------------------------------------------
  ///
  /// Loader
  ///
  /// @brief Mesh loader for wavefront .obj models
  /// @details
  ///
  /// @author <Thomas Geymayer tomgey@gmail.com>
  /// @version <1.0.0>
  class LoaderOBJ:
    public utilities::Loader<Mesh>
  {
    public:

      //------------------------------------------------------------------------
      ///
      /// @brief Constructor
      ///

      LoaderOBJ();

      //------------------------------------------------------------------------
      ///
      /// @brief Destructor
      ///

      virtual ~LoaderOBJ();

      //------------------------------------------------------------------------
      ///
      /// @brief Load a mesh from a file
      ///
      /// @return

      MeshPtr load( const core::path& file );

    private:

      typedef core::vfs::file_iterator<char> iterator_t;

      void addPositionCoord( float value );
      void addPosition( iterator_t, iterator_t const& );
      void addNormalCoord( float value );
      void addNormal( iterator_t, iterator_t const& );
      void addPositionIndex( int index );
      void addNormalIndex( int index );
      void addFace( iterator_t, iterator_t const& );
      void addMatLib( iterator_t first, iterator_t const& last );

      //------------------------------------------------------------------------
      // parsing variables

      math::Vector3f        _tmp_position;
      math::Vector3f        _tmp_normal;
      math::uint32          _tmp_position_index[4];
      math::uint32          _tmp_normal_index[4];

      math::uint8           _index_position;
      math::uint8           _index_normal;
      math::uint8           _index_position_index;
      math::uint8           _index_normal_index;

      std::vector<math::Vector3f>   _positions;
      std::vector<math::Vector3f>   _normals;
      std::vector<Vertex>           _vertices;
  };

} // namespace scene
} // namespace gewoopi

#endif /* GEWOOPI_LOADER_OBJ_HPP_ */
