//------------------------------------------------------------------------------
// renderer.h - this file is part of the gewoopi game engine
// (c) copyright 2008 Thomas Geymayer <tomgey@gmail.com>
// For conditions of distribution and use, see copyright notice in gewoopi.h
//------------------------------------------------------------------------------

#ifndef OPENGL_RENDERER_H_
#define OPENGL_RENDERER_H_

#include "video/renderer.h"

namespace gewoopi
{
namespace video
{

  class OGLRenderer: public Renderer
  {
    public:

      //------------------------------------------------------------------------

      OGLRenderer();
      virtual ~OGLRenderer();

      //------------------------------------------------------------------------
      ///
      /// @brief set new clear color
      ///
      /// @return previous clear color

      virtual math::Color3f setClearColor( const math::Color3f& color );

      // virtual methods from base class

      void loadMatrix( const math::Matrix4x4f& mat );
      void multMatrix( const math::Matrix4x4f& mat );

      void pushMatrix();
      void popMatrix();

      //------------------------------------------------------------------------
      ///
      /// @brief Must be called once before rendering
      ///
      /// @param clear_back_buffer True to clear the backbuffer.
      /// @param clear_z_buffer True to clear the z-buffer.

      void beginScene( bool clear_back_buffer = true,
                       bool clear_z_buffer    = true );

      //------------------------------------------------------------------------
      ///
      /// @brief Must be called once after finished rendering
      ///

      void endScene();

      //------------------------------------------------------------------------

      void drawPrimitives(        const PrimitiveType  type,
                                  scene::VertexArray vertices,
                                  scene::Count count );

      void drawIndexedPrimitives( const PrimitiveType  type,
                                  scene::VertexArray vertices,
                                  scene::Count count,
                                  const math::Color3f *color = 0 );

      //------------------------------------------------------------------------

      void performOperation( RenderOperation& operation );

      //------------------------------------------------------------------------
      ///
      /// @brief
      ///

      void setCullMode( const CullMode mode );

      //------------------------------------------------------------------------
      ///
      /// @brief
      ///

      void setMaterial( const Material& mat );

    private:

      unsigned int getPrimitiveType( const PrimitiveType type );
  };

} // namespace video
} // namespace gewoopi

#endif /* OPENGL_RENDERER_H_ */
