//------------------------------------------------------------------------------
// OpenGL/window.h - this file is part of the gewoopi game engine
// (c) copyright 2008 Thomas Geymayer <tomgey@gmail.com>
// For conditions of distribution and use, see copyright notice in gewoopi.h
//------------------------------------------------------------------------------

#ifndef OPENGL_WINDOW_H_
#define OPENGL_WINDOW_H_

#include <SDL/SDL.h>

#include "video/render_target.h"
#include "video/window.h"

#include "math/math.h"


namespace gewoopi
{
namespace video
{

  class OGLWindow: public Window
  {
    public:

      //------------------------------------------------------------------------
      ///
      /// @brief Create a new OpenGL window with the specified size and format.
      ///
      /// Optionally also link an input manager to the window.
      ///
      /// @param size The two dimensional size of the window
      /// @param format The surface format
      /// @param input_manager An input manager which should be notified about
      ///        key and input events received by this window

      OGLWindow( const math::Size2D<int>& size,
                 SurfaceFormat format = R8G8B8,
                 input::ManagerPtr input_manager = input::ManagerPtr() );

      //------------------------------------------------------------------------
      ///
      /// @brief Destructor
      ///

      virtual ~OGLWindow();

      //------------------------------------------------------------------------
      ///
      /// @brief create window an generate ogl context
      ///

      bool load();

      bool updateCaption( const std::string& caption );

      //------------------------------------------------------------------------
      ///
      ///
      ///

      void processMessages();

    protected:

      SDL_Surface *_screen;

      input::ManagerPtr _input_manager;
      input::DeviceId _keyboard_id;
      input::DeviceId _mouse_id;

      //------------------------------------------------------------------------
      ///
      /// @brief Initializes alle input devices
      /// Enumerates all input devices and registers them at the input manager

      void initInputDevices();
  };

} // namespace video
} // namespace gewoopi

#endif /* OPENGL_WINDOW_H_ */
