//------------------------------------------------------------------------------
// face.cpp - this file is part of the gewoopi game engine
// (c) copyright 2009 Thomas Geymayer <tomgey@gmail.com>
// For conditions of distribution and use, see copyright notice in gewoopi.h
//------------------------------------------------------------------------------

/*!
 * @file
 * @brief
 * @details
 * @author Thomas Geymayer <tomgey@gmail.com>
 * @date Date of Creation: 19.10.2009
 */

#include "std_test.hpp"

#include <gewoopi/math/face.hpp>

BOOST_AUTO_TEST_CASE( Face_Create )
{
  using gewoopi::math::Facef;
  using gewoopi::math::Vector3f;
  using gewoopi::math::Ray3f;

  const float max_close = 0.001f;

  //----------------------------------------------------------------------------
  // create a face and check some simple hits

  Vector3f p0( 0, 1, 2 );
  Vector3f p1( 1, 3, 4 );
  Vector3f p2( 4, 0, 2 );
  Vector3f normal( Vector3f(p2 - p0).cross(p1 - p0).normalize() );

  Facef face1( p0, p1, p2 );
  gewoopi::math::real32 dist = 17;

  std::cout << "Start testing..." << std::endl;

  BOOST_CHECK( face1.hit( Ray3f(p0 + normal, -normal), &dist) );
  BOOST_CHECK( face1.hit( Ray3f(p1 + normal, -normal), &dist) );
  BOOST_CHECK( face1.hit( Ray3f(p2 + normal, -normal), &dist) );
  BOOST_CHECK_CLOSE( dist, 1.f, max_close );
}
