//------------------------------------------------------------------------------
// plane.cpp - this file is part of the gewoopi game engine
// (c) copyright 2009 Thomas Geymayer <tomgey@gmail.com>
// For conditions of distribution and use, see copyright notice in gewoopi.h
//------------------------------------------------------------------------------

/*!
 * @file
 * @brief Test for the plane class.
 * @details
 * @author Thomas Geymayer <tomgey@gmail.com>
 * @date Date of Creation: 16.10.2009
 */

#include "std_test.hpp"

#include <gewoopi/math/plane.hpp>

BOOST_AUTO_TEST_CASE( Plane_Create )
{
  using gewoopi::math::Planef;
  using gewoopi::math::Vector3f;
  using gewoopi::math::Ray3f;

  const float max_close = 0.001f;

  //----------------------------------------------------------------------------
  // Let's start with a simple plane
  //

  Planef plane1( Vector3f(0,0,0), Vector3f(0,0,-1), Vector3f(1,0,-1) );

  std::cout << "Testing plane " << plane1 << std::endl;

  BOOST_CHECK_EQUAL( plane1.getNormal(), Vector3f(0,1,0) );
  BOOST_CHECK_CLOSE( plane1.getNormal().length(), 1.f, max_close );
  BOOST_CHECK_CLOSE( plane1.getD(), 0.f, max_close );

  BOOST_CHECK_CLOSE( plane1.distanceTo(Vector3f( 0, 1, 0)), 1.f, max_close);
  BOOST_CHECK_CLOSE( plane1.distanceTo(Vector3f( 0,-1, 0)),-1.f, max_close);
  BOOST_CHECK_CLOSE( plane1.distanceTo(Vector3f( 0, 0, 0)), 0.f, max_close);

  gewoopi::math::real32 dist = 17;
  Ray3f ray1( Vector3f(0,-1,0), Vector3f(1,1.1,2.2) );

  // this hit should fail, therefore dist should stay unchanged
  BOOST_CHECK( !plane1.hit( ray1, &dist ) );
  BOOST_CHECK_CLOSE( dist, 17.f, max_close );

  // now it should really hit the plane
  Ray3f ray2( Vector3f(0,1,1), Vector3f(0.7,-1.1,2.2) );
  BOOST_CHECK( plane1.hit( ray2, &dist ) );
  BOOST_CHECK_CLOSE( dist, 0.909090f, max_close );

  //----------------------------------------------------------------------------
  // and now a plane somewhere in the space...
  //

  Planef plane2( Vector3f( 129.34, -400.0,    2.01),
                 Vector3f( -12.90, 1398.2,   -1.20),
                 Vector3f(2930.19, 8045.1, 1410.22) );

  std::cout << "Testing plane " << plane2 << std::endl;

  BOOST_CHECK_EQUAL( plane2.getNormal(),
                     Vector3f(-0.37944f,-0.02836f,0.92478f) );
  BOOST_CHECK_CLOSE( plane2.getNormal().length(), 1.f, max_close );
  BOOST_CHECK_CLOSE( plane2.getD(), -35.8726f, max_close );

}
