//------------------------------------------------------------------------------
// unit_tests.cpp - this file is part of the gewoopi game engine
// (c) copyright 2008 Thomas Geymayer <tomgey@gmail.com>
// For conditions of distribution and use, see copyright notice in gewoopi.h
//------------------------------------------------------------------------------

#include "std_test.hpp"

#include <gewoopi/math/vector.hpp>
#include <gewoopi/math/matrix.hpp>

using std::cout;
using std::endl;

BOOST_AUTO_TEST_CASE( Vector3_Test )
{
  using gewoopi::math::Vector;
  using gewoopi::math::Vector3;
  using gewoopi::math::Vector3f;

  float vals_f1[3] = { 1.1, 2.2, 3.3 },
        vals_f2[3] = { 2,   4,   6   };

  Vector<float,3> vec_f1( vals_f1 ),
                  vec_f2( vals_f2 );

  int vals_i1[3]   = { 2, 4, 6       };

  Vector<int,3>   vec_i1( vals_i1 );

  Vector3f        vec3f_1(1.1,2.2,3.3),
                  vec3f_2(2,4,6);

  BOOST_CHECK_EQUAL( vec_f1, Vector3f(1.1,2.2,3.3) );
  BOOST_CHECK_EQUAL( Vector3f(1.1,2.2,3.3), vec_f1 );

  BOOST_CHECK_NE(    vec_f1, Vector3f(1.0,2.2,3.3) );
  BOOST_CHECK_NE(    vec_f1, Vector3f(1.1,2.1,3.3) );
  BOOST_CHECK_NE(    vec_f1, Vector3f(1.1,2.2,3.2) );

  BOOST_CHECK_NE(    Vector3f(1.0,2.2,3.3), vec_f1 );
  BOOST_CHECK_NE(    Vector3f(1.1,2.1,3.3), vec_f1 );
  BOOST_CHECK_NE(    Vector3f(1.1,2.2,3.2), vec_f1 );

  BOOST_CHECK_EQUAL( vec_i1, vec_f2 );
  BOOST_CHECK_EQUAL( vec_f2, vec_i1 );

  BOOST_CHECK_EQUAL( vec3f_1.lengthSqr(),   vec3f_1.x() * vec3f_1.x()
                                          + vec3f_1.y() * vec3f_1.y()
                                          + vec3f_1.z() * vec3f_1.z() );
}

BOOST_AUTO_TEST_CASE( Matrix_Test )
{
  using gewoopi::math::Matrix4x4;
  using gewoopi::math::Matrix4x4f;

  float vals1[4][4] = { { 1,  0,  0,  0},
                        { 0,  1,  0,  0},
                        { 0,  0,  1,  0},
                        { 0,  0,  0,  1} };

  float vals2[4][4] = { { 1,  2,  3,  4},
                        { 5,  6,  7,  8},
                        { 9, 10, 11, 12},
                        {13, 14, 15, 16} };

  Matrix4x4<float> mat_identity( vals1 );
  Matrix4x4<float> mat_test2( vals2 );

  BOOST_CHECK_EQUAL( mat_test2(1,1),  1 );
  BOOST_CHECK_EQUAL( mat_test2(1,2),  2 );
  BOOST_CHECK_EQUAL( mat_test2(1,3),  3 );
  BOOST_CHECK_EQUAL( mat_test2(1,4),  4 );
  BOOST_CHECK_EQUAL( mat_test2(2,1),  5 );
  BOOST_CHECK_EQUAL( mat_test2(2,2),  6 );
  BOOST_CHECK_EQUAL( mat_test2(2,3),  7 );
  BOOST_CHECK_EQUAL( mat_test2(2,4),  8 );
  BOOST_CHECK_EQUAL( mat_test2(3,1),  9 );
  BOOST_CHECK_EQUAL( mat_test2(3,2), 10 );
  BOOST_CHECK_EQUAL( mat_test2(3,3), 11 );
  BOOST_CHECK_EQUAL( mat_test2(3,4), 12 );
  BOOST_CHECK_EQUAL( mat_test2(4,1), 13 );
  BOOST_CHECK_EQUAL( mat_test2(4,2), 14 );
  BOOST_CHECK_EQUAL( mat_test2(4,3), 15 );
  BOOST_CHECK_EQUAL( mat_test2(4,4), 16 );

  BOOST_CHECK_EQUAL( mat_test2, mat_test2 * mat_identity );
  BOOST_CHECK_EQUAL( mat_test2, mat_identity * mat_test2 );

  mat_test2 *= mat_identity;

  BOOST_CHECK_EQUAL( mat_test2, Matrix4x4<float>(vals2) );

  Matrix4x4f temp( Matrix4x4f::Identity() );

  cout << temp * mat_identity << endl;

  gewoopi::math::Vector3f        vec3f_1(1.1,2.2,3.3);

  cout << Matrix4x4f::Identity() * vec3f_1 << endl;

}
